@@ -79,6 +79,9 @@ REPO_NAME_LOC            = "profiles" + "/" + REPO_NAME_FILE
 PORTAGE_PACKAGE_ATOM     = "sys-apps/portage"
 LIBC_PACKAGE_ATOM        = "virtual/libc"
 OS_HEADERS_PACKAGE_ATOM  = "virtual/os-headers"
+CVS_PACKAGE_ATOM         = "dev-vcs/cvs"
+GIT_PACKAGE_ATOM         = "dev-vcs/git"
+RSYNC_PACKAGE_ATOM       = "net-misc/rsync"
 
 INCREMENTALS             = ("USE", "USE_EXPAND", "USE_EXPAND_HIDDEN",
                            "FEATURES", "ACCEPT_KEYWORDS",
@@ -164,9 +167,12 @@ if "PORTAGE_OVERRIDE_EPREFIX" in os.environ:
 	EPREFIX = os.environ["PORTAGE_OVERRIDE_EPREFIX"]
 	if EPREFIX:
 		EPREFIX = os.path.normpath(EPREFIX)
+		if EPREFIX == os.sep:
+			EPREFIX = ""
 
 VCS_DIRS = ("CVS", "RCS", "SCCS", ".bzr", ".git", ".hg", ".svn")
 
+SUPPORTED_BINPKG_FORMATS = ("tar", "rpm")
 # ===========================================================================
 # END OF CONSTANTS -- END OF CONSTANTS -- END OF CONSTANTS -- END OF CONSTANT
 # ===========================================================================
@@ -174,6 +180,4 @@ VCS_DIRS = ("CVS", "RCS", "SCCS", ".bzr", ".git", ".hg", ".svn")
 # Private constants for use in conditional code in order to minimize the diff
 # between branches.
 _DEPCLEAN_LIB_CHECK_DEFAULT = True
-_ENABLE_REPO_NAME_WARN  = False
-_ENABLE_SET_CONFIG      = False
-_ENABLE_INHERIT_CHECK   = False
+_ENABLE_SET_CONFIG      = True
